﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Serialization;
using Mozlite.Data;

namespace Mozlite.Components
{
    /// <summary>
    /// 多语言配置基类。
    /// </summary>
    /// <typeparam name="TConfiguration">配置类型。</typeparam>
    [Serializable]
    public abstract class BaseResourceConfiguration<TConfiguration> : BaseConfiguration<TConfiguration>, IResourceConfiguration
        where TConfiguration : BaseResourceConfiguration<TConfiguration>, new()
    {
        /// <summary>
        /// 初始化类<see cref="BaseResourceConfiguration&lt;TConfiguration&gt;"/>。
        /// </summary>
        public BaseResourceConfiguration() {
            CultureName = AppContext.Current.CultureName;
        }

        /// <summary>
        /// 当前资源名称。
        /// </summary>
        [XmlIgnore]
        public string CultureName { get; set; }

        /// <summary>
        /// 获取缓存键。
        /// </summary>
        /// <returns>返回缓存键。</returns>
        protected override string GetCacheKey()
        {
            return "ResourceConfigurations[{1}][{0}]".FormatWith(ConfigurationKey, CultureName);
        }

        /// <summary>
        /// 从数据库中加载配置。
        /// </summary>
        /// <param name="configuration">加载的配置实例对象。</param>
        public override void LoadConfiguration(ref TConfiguration configuration)
        {
            using (var reader = Database.ExecuteReader("SELECT * FROM moz_resources_configurations WHERE [Key] = @Key AND [CultureName] = @CultureName", new { Key = ConfigurationKey, CultureName = CultureName }))
            {
                if (reader.Read())
                {
                    configuration = Cores.Deserialize<TConfiguration>(reader.GetString("SettingsXML"));
                    configuration.CreateDate = reader.GetDateTime("CreateDate");
                    configuration.LastUpdateDate = reader.GetDateTime("LastUpdateDate");
                }
                reader.Close();
            }
        }

        /// <summary>
        /// 获取配置。
        /// </summary>
        /// <param name="cultureName">当前语言。</param>
        /// <returns>返回配置实例对象。</returns>
        public static TConfiguration GetConfiguration(string cultureName) {
            TConfiguration configuration = null;
            using (var reader = Database.ExecuteReader("SELECT * FROM moz_resources_configurations WHERE [Key] = @Key AND [CultureName] = @CultureName", new { Key = ConfigurationKey, CultureName = cultureName }))
            {
                if (reader.Read())
                {
                    configuration = Cores.Deserialize<TConfiguration>(reader.GetString("SettingsXML"));
                    configuration.CreateDate = reader.GetDateTime("CreateDate");
                    configuration.LastUpdateDate = reader.GetDateTime("LastUpdateDate");
                }
                reader.Close();
            }
            return configuration;
        }

        /// <summary>
        /// 保存当前配置。
        /// </summary>
        /// <returns>返回保存结果。</returns>
        public override bool Save()
        {
            if (Database.ExecuteNonQuery(@"
IF EXISTS(SELECT 1 FROM moz_resources_configurations WHERE [Key] = @Key AND [CultureName] = @CultureName) BEGIN
    UPDATE moz_resources_configurations SET [SettingsXML] = @SettingsXML, [LastUpdateDate] = GETDATE() WHERE [Key] = @Key AND [CultureName] = @CultureName;
END ELSE BEGIN
    INSERT INTO moz_resources_configurations([Key], [CultureName], [CreateDate], [LastUpdateDate], [SettingsXML])VALUES(@Key, @CultureName, GETDATE(), GETDATE(), @SettingsXML);
END", new { SettingsXML = Cores.Serialize((TConfiguration)this), Key = ConfigurationKey, CultureName = CultureName }))
            {
                Caches.Remove(this.GetCacheKey());
                return true;
            }
            return false;
        }
    }
}
