﻿/*********************************************************
 * 开发人员：小兵
 * 创建时间：2012-10-9 10:24:00
 * 描述说明：
 * 
 * 更改历史：
 * 
 * *******************************************************/
namespace Mozlite.Data.SqlCE
{
    using System;
    using System.Web;
    using System.Data;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using System.Diagnostics;
    using Mozlite.Data.Components.Structures;

    /// <summary>
    /// SQL数据库。
    /// </summary>
    public class SqlCeDatabase : Database<SqlCeHelper>
    {
        /// <summary>
        /// 初始化类<see cref="SqlCeDatabase"/>。
        /// </summary>
        /// <param name="helper">数据库辅助接口实例。</param>
        public SqlCeDatabase(SqlCeHelper helper)
            : base(helper)
        { }
        
        /// <summary>
        /// 通过名称获取表格实例。
        /// </summary>
        /// <param name="name">名称。</param>
        /// <returns>如果存在返回表格实例，否则返回<c>null</c>。</returns>
        public override ITable GetTable(string name)
        {
            if (name == null)
                throw new ArgumentNullException("name");

            ITable table = null;

            // get name in correct casing
            name = SqlHelper.ExecuteScalar<string>("SELECT name FROM sys.tables WHERE name=@name",
                                                          SqlHelper.CreateParameter("name", name));
            if (name != null)
            {
                table = new Table(name);

                using (var reader = SqlHelper.ExecuteReader(
                        @"SELECT c.name AS Name, st.name AS DataType, c.max_length, c.is_nullable, c.is_identity
                          FROM sys.tables AS t
                            JOIN sys.columns AS c ON t.object_id = c.object_id
                            JOIN sys.schemas AS s ON s.schema_id = t.schema_id
                            JOIN sys.types AS ty ON ty.user_type_id = c.user_type_id
                            JOIN sys.types st ON ty.system_type_id = st.user_type_id
                          WHERE t.name = @name
                          ORDER BY c.column_id", SqlHelper.CreateParameter("name", name)))
                {
                    while (reader.Read())
                    {
                        table.AddField(table.CreateField(reader.GetString("Name"), GetType(reader)));
                    }
                }
            }

            return table;
        }
        
        /// <summary>
        /// 保存表格。
        /// </summary>
        /// <param name="table">要保存的表格实例对象。</param>
        public override void SaveTable(ITable table)
        {
            if (table == null)
                throw new ArgumentNullException("table");

            ITable oldTable = GetTable(table.Name);

            // 如果不存在则创建，否则更新。
            if (oldTable == null)
            {
                CreateTable(table);
            }
            else
            {
                foreach (IField field in table)
                {
                    // 旧的数据库中如果没有该列，则新建该列
                    if (oldTable.FindField(field.Name)==null)
                    {
                        CreateColumn(table, field);
                    }
                }
            }
        }

        /// <summary>
        /// 新建表格。
        /// </summary>
        /// <param name="table">表格对象实例。</param>
        protected virtual void CreateTable(ITable table)
        {
            Debug.Assert(table != null);

            // create enumerator and check for first field
            IEnumerator<IField> fieldEnumerator = table.GetEnumerator();
            bool hasNext = fieldEnumerator.MoveNext();
            if(!hasNext)
                throw new ArgumentException("The table must contain at least one field.");

            // create query
            StringBuilder createTableQuery = new StringBuilder();
            createTableQuery.AppendFormat("CREATE TABLE [{0}] (", SqlHelper.EscapeString(table.Name));

            // add fields
            while (hasNext)
            {
                // add field name and type
                IField field = fieldEnumerator.Current;
                createTableQuery.Append('[').Append(field.Name).Append(']');
                createTableQuery.Append(' ').Append(GetDatabaseType(field));

                // append comma if a following field exists
                hasNext = fieldEnumerator.MoveNext();
                if (hasNext)
                {
                    createTableQuery.Append(',');
                }
            }

            // close CREATE TABLE x (...) bracket
            createTableQuery.Append(')');

            // execute query
            try
            {
                SqlHelper.ExecuteNonQuery(createTableQuery.ToString());
            }
            catch (Exception executeException)
            {
                throw new Exception(String.Format("Could not create table '{0}'.", table), executeException);
            }
        }

        /// <summary>
        /// 新建列。
        /// </summary>
        /// <param name="table">表格实例对象。</param>
        /// <param name="field">列对象实例。</param>
        protected virtual void CreateColumn(ITable table, IField field)
        {
            Debug.Assert(table != null && field != null);

            StringBuilder addColumnQuery = new StringBuilder();
            addColumnQuery.AppendFormat("ALTER TABLE [{0}] ADD [{1}] {2}",
                                        SqlHelper.EscapeString(table.Name),
                                        SqlHelper.EscapeString(field.Name),
                                        SqlHelper.EscapeString(GetDatabaseType(field)));
            try
            {
                SqlHelper.ExecuteNonQuery(addColumnQuery.ToString());
            }
            catch (Exception executeException)
            {
                throw new Exception(String.Format("Could not create column '{0}' in table '{1}'.",
                                                         field, table.Name), executeException);
            }
        }

        /// <summary>
        /// 将数据库类型转换为.NET数据类型。
        /// </summary>
        /// <param name="dataTypeReader">数据库读取器。</param>
        /// <returns>返回类型实例。</returns>
        protected virtual Type GetType(IDataReader dataTypeReader)
        {
            string typeName = dataTypeReader.GetString("DataType");

            switch (typeName)
            {
                case "bit":                 return typeof(bool);
                case "tinyint":             return typeof(byte);
                case "datetime":            return typeof(DateTime);
                // TODO: return different decimal type according to field precision
                case "decimal":             return typeof(decimal);
                case "uniqueidentifier":    return typeof(Guid);
                case "smallint":            return typeof(Int16);
                case "int":                 return typeof(Int32);
                case "bigint":              return typeof(Int64);
                case "ntext":
                case "nvarchar":            return typeof(string);
                default:
                    throw new ArgumentException("Cannot convert database type '{0}' to a .Net type.".FormatWith(typeName));
            }
        }

        /// <summary>
        /// 获取列的数据库类型字符串。
        /// </summary>
        /// <param name="field">列实例对象。</param>
        /// <returns>返回列的数据库类型字符串。</returns>
        protected virtual string GetDatabaseType(IField field)
        {
            StringBuilder fieldBuilder = new StringBuilder();

            fieldBuilder.Append(GetDatabaseTypeName(field));
            fieldBuilder.Append(field.HasProperty(FieldProperties.Identity) ? " IDENTITY(1,1)" : String.Empty);
            fieldBuilder.Append(field.HasProperty(FieldProperties.NotNull) ? " NOT NULL" : " NULL");

            return fieldBuilder.ToString();
        }

        /// <summary>
        /// 获取数据库类型字符串，未添加列属性。
        /// </summary>
        /// <param name="field">列实例对象。</param>
        /// <returns>返回列的数据库类型字符串。</returns>
        protected virtual string GetDatabaseTypeName(IField field)
        {
            switch (field.DataType.FullName)
            {
                case "System.Boolean":  return "bit";
                case "System.Byte":     return "tinyint";
                case "System.DateTime": return "datetime";
                case "System.Decimal":  return "decimal(28)";
                case "System.Double":   return "decimal(15)";
                case "System.Single":   return "decimal(7)";
                case "System.Guid":     return "uniqueidentifier";
                case "System.Int16":    return "smallint";
                case "System.Int32":    return "int";
                case "System.Int64":    return "bigint";;
                case "System.String":
                    return field.Size == 0 ? "ntext" : "nvarchar({0})".FormatWith(field.Size);
                default:
                    throw new ArgumentException("Cannot convert '{0}' to a database type.".FormatWith(field));
            }
        }
    }
}
